.comma-delimited { 
  background: url(bg.jpg) no-repeat, url(bg.png) repeat-x top left, url(bg);
  text-shadow: -1px -1px 1px red, 6px 5px 5px yellow;
  -moz-box-shadow: 0pt 0pt 2px rgba(255, 255, 255, 0.4) inset,
    0pt 4px 6px rgba(255, 255, 255, 0.4) inset;
}
@font-face {
  font-family: Headline;
  src: local(Futura-Medium),
       url(fonts.svg#MyGeometricModern) format("svg");
}
.other {
  -moz-transform: translate(0, 11em) rotate(-90deg);
}
p:not([class*="lead"]) {
  color: black; 
}

input[type="text"].class#id[attr=32]:not(1) {
  color: white;
}

div#id.class[a=1][b=2].class:not(1) {
  color: white;
}

ul.comma > li:not(:only-child)::after { 
  color: white;
}

ol.comma > li:nth-last-child(2)::after {
  color: white;
}

li:nth-child(4n+1),
li:nth-child(-5n),
li:nth-child(n+1),
li:nth-child(-n+2) {
  color: white;
}

a[href^="http://"] {
  color: black;
}

a[href$="http://"] {
  color: black;
}

p::before {
  color: black;
}
